
# q-expansion of newform 1920.2.a.b, downloaded from the LMFDB on 26 December 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 1920
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0], [-1], [-1], [-2], [-6], [-2], [-2], [-2], [6], [10], [-4], [2], [6], [0], [6], [-10], [6], [6], [-16], [8], [10], [0], [-12], [6], [10], [10], [10], [4], [6], [-18], [2], [22], [-2], [-14], [18], [0], [18], [-12], [-6], [18], [18], [-18], [12], [-22], [-2], [-4], [10], [-2], [8], [-26], [-26], [0], [18], [18], [-2], [14], [-6], [-4], [-14], [18], [8], [-6], [8], [-24], [-6], [14], [30], [18], [12], [22], [-10], [4], [-22], [-38], [18], [34], [-30], [14], [-34], [6], [10], [38], [-28], [18], [4], [24], [-6], [-22], [2], [6], [24], [24], [-26], [-14], [-34], [-34], [-22], [34], [-16], [-26], [16], [18], [4], [-42], [46], [18], [-4], [-10], [36], [10], [26], [-26], [6], [-6], [16], [18], [-12], [42], [22], [26], [38], [-6], [30], [-8], [26], [10], [38], [16], [38], [-22], [-2], [-50], [-28], [18], [2], [-26], [6], [-48], [-18], [42], [-26], [18], [2], [-4], [38], [-36], [22], [-2], [2], [18], [-46], [14], [20], [-46], [28], [-20], [-4], [58], [2], [-38], [32], [-2], [22], [-54], [46], [-26], [60], [6], [38], [54], [-18], [-2], [48], [-14], [56], [-10], [-2], [42], [58], [30], [42], [46], [-26], [-26], [26], [-38], [34], [36], [-26], [-28], [26], [0], [-22], [-30], [-32], [54], [30], [30], [-42], [22], [-6], [60], [6], [-62], [38], [66], [-32], [-60], [6], [-18], [66], [-14], [-22], [-60], [-68], [-2], [-30], [-18], [50], [6], [-2], [-4], [30], [-10], [56], [-10], [-18], [-32], [6], [-30], [14], [14], [-12], [18], [56], [-42], [18], [54], [22], [0], [4], [70], [18], [14], [-42], [44], [26], [30], [-46], [-46], [-66], [-14], [26], [-70], [50], [-6], [-42], [-68], [-38], [-14], [-26], [-8], [46], [26], [-18], [14], [-6], [54], [-80], [-38], [-74], [32], [50], [-80], [26], [-54], [4], [22], [10], [22], [18], [-32], [58], [-58], [-20], [-20], [-14], [-66], [44], [-66], [-54], [-56], [-42], [-14], [26], [-14], [76], [42], [6], [8], [18], [-14], [0], [-36], [-74], [-30], [-4], [30], [12], [42], [-30], [-22], [14], [28], [-6], [54], [-78], [-28], [-62], [-82], [-22], [50], [42], [46], [14], [50], [-34], [16], [30], [22], [70], [-30], [24], [12], [-10], [76], [-34], [78], [-22], [42], [-22], [6], [-30], [8], [66], [2], [-34], [-12], [-4], [-50], [-22], [-30], [34], [46], [62], [30], [72], [-14], [6], [-58], [-58], [-42], [54], [-18], [24], [34], [14], [18], [-34], [-10], [-14], [58], [50], [0], [-2], [-46], [-76], [-46], [-22], [82], [42], [54], [22], [-82], [14], [94], [-20], [-62], [-48], [62], [22], [90], [14], [-80], [64], [58], [56], [-54], [6], [-30], [-34], [-18], [82], [46], [-30], [40], [46], [-34], [-44], [18], [26], [106], [64], [-38], [-38], [34], [-80], [62], [-18], [-34], [90], [54], [-50], [-18], [-62], [18], [-36], [26], [-98], [60]]
